function varargout=alldemos(varargin)
  p = inputParser; 
  p.KeepUnmatched=true; 
  p.addParameter('stop',false,@islogical);
  p.addParameter('graphic',true,@islogical);
  p.addParameter('logfile','',@ischar);
  p.addParameter('tpause',2,@isscalar);
  p.parse(varargin{:});
  stop=p.Results.stop;
  logfile=p.Results.logfile;
  graphic=p.Results.graphic;
  R=p.Results;
  if ~isempty(logfile)
    fid=fopen(logfile,'a');
    if fid==-1, error('Unable to open file %s\n',logfile);end
    fids=[1,fid]; % 1 standard output
  else
    fids=1;
  end
  
  C=clock;
  strdate=sprintf('%s at %02d:%02d:%02d\n',date,C(4),C(5),round(C(6)));
  gmsh_version=fc_oogmsh.gmsh.version();
  
  linesep=fc_tools.utils.line_text_delimiter();
  fprintf(linesep)
  fprintf(linesep)
  vfprintf(fids,linesep)
  vfprintf(fids,'Starting alldemos with:\n')
  [Soft,Release]=fc_tools.sys.getSoftware();
  vfprintf(fids,'   %10s : %s\n',Soft,Release)
  vfprintf(fids,'   %10s : %s\n','gmsh',gmsh_version)
  vfprintf(fids,'   %10s : %s','date',strdate)
  vfprintf(fids,linesep)
  
  MSHformats=fc_oogmsh.gmsh.getMSHfile_formats();
  
  geofiles=repmat(struct('name',[],'d',[],'N',[],'np',5),1,3);
  geofiles(1).name='condenser.geo';
  geofiles(1).d=2;
  geofiles(1).N=10;
  geofiles(2).name='demisphere5.geo';
  geofiles(2).d=2;
  geofiles(2).N=6;
  geofiles(3).name='cylinderkey.geo';
  geofiles(3).d=3;
  geofiles(3).N=6;
  
  titles={};valid=[];expected_valid=[];
  for i=1:length(geofiles)
    for j=1:length(MSHformats)
      resume=sprintf('MSH format=%s, geofile=''%s'', d=%d, N=%d, no partition',MSHformats{j},geofiles(i).name,geofiles(i).d,geofiles(i).N);
      fun=@() fc_oogmsh.demos.gdemo('geofile',geofiles(i).name,'d',geofiles(i).d,'N',geofiles(i).N,'MshFileVersion',MSHformats{j},'graphic',graphic);
      titles{end+1}=sprintf('gdemo with %s and graphic=%d',resume,graphic);
      valid(end+1)=check_run(fun,stop);
      expected_valid(end+1)=fc_oogmsh.gmsh.isGoodMshFormatVersion(MSHformats{j});
      s=strvalid(valid(end),expected_valid(end));
      fprintf(linesep)
      vfprintf(fids,'[fc-oogmsh] %s \n  ->  running %s\n',s,titles{end});
      if valid(end)~=expected_valid(end), vfprintf(fids,'   RESULT NOT EXPECTED : FAILED...FAILED\n');end
      fprintf(linesep)
      pause(R.tpause)
    end
  end
  
  for i=1:length(geofiles)
    for j=1:length(MSHformats)
      resume=sprintf('MSH format=%s, geofile=''%s'', d=%d, N=%d, np=%d partitions',MSHformats{j},geofiles(i).name,geofiles(i).d,geofiles(i).N,geofiles(i).np);
      fun=@() fc_oogmsh.demos.gpdemo01('geofile',geofiles(i).name,'d',geofiles(i).d,'N',geofiles(i).N,'np',geofiles(i).np,'MshFileVersion',MSHformats{j},'graphic',graphic);
      titles{end+1}=sprintf('gpdemo01 with %s and graphic=%d',resume,graphic);
      valid(end+1)=check_run(fun,stop);
      expected_valid(end+1)=fc_oogmsh.gmsh.isGoodMshFormatVersionForPartionedMesh(MSHformats{j});
      s=strvalid(valid(end),expected_valid(end));
      %if valid(end)==0,s='FAILED';else,s='OK';end
      fprintf(linesep)
      vfprintf(fids,'[fc-oogmsh] %s \n  ->  running %s\n',s,titles{end});
      if valid(end)~=expected_valid(end), vfprintf(fids,'   RESULT NOT EXPECTED : FAILED...FAILED\n');end
      fprintf(linesep)
      pause(R.tpause)
      %close all
      fun=@() fc_oogmsh.demos.gpdemo02('geofile',geofiles(i).name,'d',geofiles(i).d,'N',geofiles(i).N,'np',geofiles(i).np,'MshFileVersion',MSHformats{j},'graphic',graphic);
      titles{end+1}=sprintf('gpdemo02 with %s and graphic=%d',resume,graphic);
      valid(end+1)=check_run(fun,stop);
      expected_valid(end+1)=fc_oogmsh.gmsh.isGoodMshFormatVersionForPartionedMesh(MSHformats{j});
      s=strvalid(valid(end),expected_valid(end));
      fprintf(linesep)
      vfprintf(fids,'[fc-oogmsh] %s \n  ->  running %s\n',s,titles{end});
      if valid(end)~=expected_valid(end), vfprintf(fids,' -> RESULT NOT EXPECTED : FAILED...FAILED\n');end
      fprintf(linesep)
    end
  end
  
  
  ndemos=length(valid);
  if nargout>=1,varargout{1}=valid;end
  if nargout==2,varargout{2}=expected_valid;end  
  fprintf(linesep)
  fprintf('            RESUME\n')
  fprintf('   %10s : %s\n',Soft,Release)
  fprintf('   %10s : %s\n','gmsh',gmsh_version)
  fprintf('   %10s : %s','date',strdate)
  fprintf(linesep)
  OK=true;
  for i=1:ndemos
    s=strvalid(valid(i),expected_valid(i));
%      if valid(i)==0
%        if valid(i)==expected_valid(i)
%          s='FAILED (expected)';
%        else
%          s='FAILED (not expected)';
%        end
%      else
%        s='OK';
%      end
    fprintf('[fc-oogmsh] %25s : running %s\n',s,titles{i});
    if valid(i)~=expected_valid(i) 
      fprintf(' ->  RESULT NOT EXPECTED : FAILED...FAILED\n');
      OK=false;
    end
  end
  fprintf(linesep)
  if length(fids)==2,fclose(fid);end
  if OK
    fprintf('[fc-oogmsh] ALL RESULTS ARE THOSE EXPECTED\n')
  else
    fprintf('[fc-oogmsh] AT LEAST ONE RESULT IS NOT EXPECTED : FAILED...FAILED\n')
    %error('[fc-oogmsh] AT LEAST ONE RESULT IS NOT EXPECTED : FAILED...FAILED')
  end
  fprintf(linesep)
  close all
end

function vfprintf(fids,varargin)
  for i=1:length(fids)
    fprintf(fids(i),varargin{:});
  end
end

function valid=check_run(fun,stop)
  try
    valid=1;
    fun();
  catch E
    valid=0;
    if stop
      fprintf(1,'The identifier was:\n%s',E.identifier);
      fprintf(1,'There was an error! The message was:\n%s\n',E.message);
      error('[fc-oogmsh] Stop! running %s FAILED!',fc_tools.utils.fun2str(fun))
    else
      fprintf(1,'The identifier was:\n%s',E.identifier);
      fprintf(1,'There was an error! The message was:\n%s\n',E.message);
      warning('[fc-oogmsh] Running %s FAILED!',fc_tools.utils.fun2str(fun))
    end
  end
end  

function s=strvalid(valid,expected_valid)
  if valid==0
    if valid==expected_valid
      s='FAILED (expected)';
    else
      s='FAILED (not expected)';
    end
  else
    s='OK';
  end
end
