function elmt_list=elm_type_list()
% FUNCTION gmsh.elm_type_list
%   Returns a cell array of a structure to obtain informations on 'elm_type' of gmsh.
%
% OUTPUT:
%   elmt_list: cell array such that the i-th elm_type informations are given by
%      elmt_list{i}.elm_type: 
%         number of the element -> i,
%      elmt_list{i}.desc: 
%         description of the element as a string (coming from gmsh manual),
%      elmt_list{i}.nb_nodes: 
%         number of nodes,
%      elmt_list{i}.order: 
%         order of the element,
%      elmt_list{i}.incomplete: 
%         true if incomplete element, false otherwise,
%      elmt_list{i}.d: 
%         minimal space dimension or d-simplex (point, line, triangle, 
%         tetrahedron)
%      elmt_list{i}.geo: 
%         'point', 'line', 'triangle', 'quadrangle', 'tetrahedron', 'prism' or 'pyramid'
%      elmt_list{i}.define:
%         Value given by a #define in Common/GmshDefines.h, 
%           Geo/MTriangle.h (lines 391 -> 409)
%           Geo/MTetrahedron.h (lines 465 > 483)
%           Geo/MQuadrangle.h (lines 516 -> 535)
%           Geo/MPyramid.h (lines 322 -> 339)
%           Geo/MPrism.h (lines 597 -> 626)
%           Geo/MLine.h (lines 259 -> 268)
%
% USAGE:
%   elmt_list=elm_type_list()
%
% Copyright (C) 2017-2023 F. Cuvelier
% see Common/GmshDefines.h, 
%     Geo/MTriangle.h (lines 391 -> 409)
%     Geo/MTetrahedron.h (lines 465 > 483)
%     Geo/MQuadrangle.h (lines 516 -> 535)
%     Geo/MPyramid.h (lines 322 -> 339)
%     Geo/MPrism.h (lines 597 -> 626)
%     Geo/MLine.h (lines 259 -> 268)

  settype=@(sdefine,n,s,nn,order,inc,d,geo) struct('define',sdefine,'elm_type',n,'desc',s,'nb_nodes',nn,'order',order,'incomplete',inc,'d',d,'geo',geo);
  elmt_list=cell(1,140); % 140 is the biggest elm_type in gmsh src 4.4.1
  elmt_list{1}=settype('MSH_LIN_2',1,'2-node line',2,1,false,1,'line');
  elmt_list{2}=settype('MSH_TRI_3',2,'3-node triangle',3,1,false,2,'triangle');
  elmt_list{3}=settype('MSH_QUA_4',3,'4-node quadrangle',4,1,false,2,'quadrangle');
  elmt_list{4}=settype('MSH_TET_4',4,'4-node tetrahedron',4,1,false,3,'tetrahedron');
  elmt_list{5}=settype('MSH_HEX_8',5,'8-node hexahedron',8,1,false,3,'hexahedron');
  elmt_list{6}=settype('MSH_PRI_6',6,'6-node prism',6,1,false,3,'prism');
  elmt_list{7}=settype('MSH_PYR_5',7,'5-node pyramid',5,1,false,3,'pyramid');
  elmt_list{8}=settype('MSH_LIN_3',8,'3-node second order line (2 nodes associated with the vertices and 1 with the edge)',3,2,false,1,'line');
  elmt_list{9}=settype('MSH_TRI_6',8,'6-node second order triangle (3 nodes associated with the vertices and 3 with the edges)',6,2,false,2,'triangle'); 
  elmt_list{10}=settype('MSH_QUA_9',10,'9-node second order quadrangle (4 nodes associated with the vertices, 4 with the edges and 1 with the face)',9,2,false,2,'quadrangle');
  elmt_list{11}=settype('MSH_TET_10',11,'10-node second order tetrahedron (4 nodes associated with the vertices and 6 with the edges)',10,2,false,3,'tetrahedron');
  elmt_list{12}=settype('MSH_HEX_27',12,'27-node second order hexahedron (8 nodes associated with the vertices, 12 with the edges, 6 with the faces and 1 with the volume)',27,2,false,3,'hexahedron');
  elmt_list{13}=settype('MSH_PRI_18',13,'18-node second order prism (6 nodes associated with the vertices, 9 with the edges and 3 with the quadrangular faces)',18,2,false,3,'prism');
  elmt_list{14}=settype('MSH_PYR_14',14,'14-node second order pyramid (5 nodes associated with the vertices, 8 with the edges and 1 with the quadrangular face)',14,2,false,3,'pyramid');
  elmt_list{15}=settype('MSH_PNT',15,'1-node point',1,1,false,0,'point');
  elmt_list{16}=settype('MSH_QUA_8',16,'8-node second order quadrangle (4 nodes associated with the vertices and 4 with the edges)',8,2,false,2,'quadrangle');
  elmt_list{17}=settype('MSH_HEX_20',17,'20-node second order hexahedron (8 nodes associated with the vertices and 12 with the edges)',20,2,false,3,'hexahedron');
  elmt_list{18}=settype('MSH_PRI_15',18,'15-node second order prism (6 nodes associated with the vertices and 9 with the edges)',15,2,false,3,'prism');
  elmt_list{19}=settype('MSH_PYR_13',19,'13-node second order pyramid (5 nodes associated with the vertices and 8 with the edges)',15,2,false,3,'pyramid');
  
  elmt_list{20}=settype('MSH_TRI_9',20,'9-node third order incomplete triangle (3 nodes associated with the vertices, 6 with the edges)',9,3,true,2,'triangle');
  elmt_list{21}=settype('MSH_TRI_10',21,'10-node third order triangle (3 nodes associated with the vertices, 6 with the edges, 1 with the face)',10,3,false,2,'triangle');
  elmt_list{22}=settype('MSH_TRI_12',22,'12-node fourth order incomplete triangle (3 nodes associated with the vertices, 9 with the edges)',12,4,true,2,'triangle');
  elmt_list{23}=settype('MSH_TRI_15',23,'15-node fourth order triangle (3 nodes associated with the vertices, 9 with the edges, 3 with the face)',15,4,false,2,'triangle');
  elmt_list{24}=settype('MSH_TRI_15I',24,'15-node fifth order incomplete triangle (3 nodes associated with the vertices, 12 with the edges)',15,5,true,2,'triangle');
  elmt_list{25}=settype('MSH_TRI_21',25,'21-node fifth order complete triangle (3 nodes associated with the vertices, 12 with the edges, 6 with the face)',21,5,false,2,'triangle');
  
  elmt_list{26}=settype('MSH_LIN_4',26,'4-node third order edge (2 nodes associated with the vertices, 2 internal to the edge)',4,3,false,1,'line'); %dim 1 or 2?
  elmt_list{27}=settype('MSH_LIN_5',27,'5-node fourth order edge (2 nodes associated with the vertices, 3 internal to the edge)',5,4,false,1,'line');%dim 1 or 2?
  elmt_list{28}=settype('MSH_LIN_6',28,'6-node fifth order edge (2 nodes associated with the vertices, 4 internal to the edge)',6,5,false,1,'line');%dim 1 or 2?
  
  elmt_list{29}=settype('MSH_TET_20',29,'20-node third order tetrahedron (4 nodes associated with the vertices, 12 with the edges, 4 with the faces)',20,3,false,3,'tetrahedron');
  elmt_list{30}=settype('MSH_TET_35',30,'35-node fourth order tetrahedron (4 nodes associated with the vertices, 18 with the edges, 12 with the faces, 1 in the volume)',35,4,false,3,'tetrahedron');
  elmt_list{31}=settype('MSH_TET_56',31,'56-node fifth order tetrahedron (4 nodes associated with the vertices, 24 with the edges, 24 with the faces, 4 in the volume)',56,5,false,3,'tetrahedron');
  elmt_list{32}=settype('MSH_TET_22',22,'22-node x order tetrahedron (4 nodes associated with the vertices, 12 internal to the edges, 4 internal to the faces, 1 internal to the volume)',20,3,false,3,'tetrahedron');
  elmt_list{33}=settype('MSH_TET_28',28,'28-node x order tetrahedron (4 nodes associated with the vertices, 12 internal to the edges, 4 internal to the faces, 1 internal to the volume)',20,3,false,3,'tetrahedron');
  
  elmt_list{42}=settype('MSH_TRI_28',42,'28-node 6-th order triangle (3 nodes associated with the vertices, x with the edges, x with the face)',28,6,false,2,'triangle');
  elmt_list{43}=settype('MSH_TRI_36',43,'36-node 7-th order triangle (3 nodes associated with the vertices, x with the edges, x with the face)',36,7,false,2,'triangle');
  elmt_list{44}=settype('MSH_TRI_45',44,'45-node 8-th order triangle (3 nodes associated with the vertices, x with the edges, x with the face)',45,8,false,2,'triangle');
  elmt_list{45}=settype('MSH_TRI_55',45,'55-node 9-th order triangle (3 nodes associated with the vertices, x with the edges, x with the face)',55,9,false,2,'triangle');
  elmt_list{46}=settype('MSH_TRI_66',46,'66-node 10-th order triangle (3 nodes associated with the vertices, x with the edges, x with the face)',66,10,false,2,'triangle');
  
  elmt_list{62}=settype('MSH_LIN_7',62,'7-node 6-th order edge (2 nodes associated with the vertices, 5 internal to the edge)',7,6,false,1,'line');
  elmt_list{63}=settype('MSH_LIN_8',63,'8-node 7-th order edge (2 nodes associated with the vertices, 6 internal to the edge)',8,7,false,1,'line');
  elmt_list{64}=settype('MSH_LIN_9',64,'9-node 8-th order edge (2 nodes associated with the vertices, 7 internal to the edge)',9,8,false,1,'line');
  elmt_list{65}=settype('MSH_LIN_10',65,'10-node 9-th order edge (2 nodes associated with the vertices, 8 internal to the edge)',10,9,false,1,'line');
  elmt_list{66}=settype('MSH_LIN_11',66,'11-node 10-th order edge (2 nodes associated with the vertices, 9 internal to the edge)',11,10,false,1,'line');
  
  elmt_list{71}=settype('MSH_TET_84',71,'84-node 6-th order tetrahedron (4 nodes associated with the vertices, x internal to the edges, x internal to the faces, x internal to the volume)',84,6,false,3,'tetrahedron');
  elmt_list{72}=settype('MSH_TET_120',72,'120-node 7-th order tetrahedron (4 nodes associated with the vertices, x internal to the edges, x internal to the faces, x internal to the volume)',120,7,false,3,'tetrahedron');
  elmt_list{73}=settype('MSH_TET_165',73,'165-node 8-th order tetrahedron (4 nodes associated with the vertices, x internal to the edges, x internal to the faces, x internal to the volume)',165,8,false,3,'tetrahedron');
  elmt_list{74}=settype('MSH_TET_220',74,'220-node 9-th order tetrahedron (4 nodes associated with the vertices, x internal to the edges, x internal to the faces, x internal to the volume)',220,9,false,3,'tetrahedron');
  elmt_list{75}=settype('MSH_TET_286',75,'286-node 10-th order tetrahedron (3 nodes associated with the vertices, x internal to the edges, x internal to the faces, x internal to the volume)',286,10,false,3,'tetrahedron');
  
  elmt_list{92}=settype('MSH_HEX_64',92,'64-node third order hexahedron (8 nodes associated with the vertices, 24 with the edges, 24 with the faces, 8 in the volume)',64,3,false,3,'hexahedron');
  elmt_list{93}=settype('MSH_HEX_125',93,'125-node fourth order hexahedron (8 nodes associated with the vertices, 36 with the edges, 54 with the faces, 27 in the volume)',125,4,false,3,'hexahedron'); 
  elmt_list{94}=settype('MSH_HEX_216',94,'216-node 5-th order hexahedron (8 nodes associated with the vertices, x with the edges, x with the faces, x in the volume)',125,5,false,3,'hexahedron'); 
  elmt_list{95}=settype('MSH_HEX_343',95,'343-node 6-th order hexahedron (8 nodes associated with the vertices, x with the edges, x with the faces, x in the volume)',343,6,false,3,'hexahedron');
  elmt_list{96}=settype('MSH_HEX_512',96,'512-node 7-th order hexahedron (8 nodes associated with the vertices, x with the edges, x with the faces, x in the volume)',512,7,false,3,'hexahedron');
  elmt_list{97}=settype('MSH_HEX_729',97,'729-node 8-th order hexahedron (8 nodes associated with the vertices, x with the edges, x with the faces, x in the volume)',729,8,false,3,'hexahedron');
  elmt_list{98}=settype('MSH_HEX_1000',98,'1000-node 9-th order hexahedron (8 nodes associated with the vertices, x with the edges, x with the faces, x in the volume)',1000,9,false,3,'hexahedron');
  
end
