function plotPartitionElements(Gh,varargin)
  assert(strcmp(class(Gh),'fc_oogmsh.ooGmsh2'),'First parameter must be a <fc_oogmsh.ooGmsh2> object.\b  -> 1st parameter is a %s object\n',class(Gh))
  if ~Gh.partitionnedfile, return;end
  if ~fc_oogmsh.isgraphics(), return;end
  p = inputParser;
  p.addParameter('entityDim',Gh.d:-1:0);
  p.parse(varargin{:});
  sElts=Gh.get_sElts_by_entityDim(Gh.d);
  tags=unique(sElts.part_lab(:,1))'; % partition tags
  n=length(tags);
  if n==0
    return;
  end
  Colors=fc_tools.graphics.selectColors(n);
  for entityDim=p.Results.entityDim
    hh=figure();
    hold on
    if (entityDim<=1) 
      %Gh.plotmesh(entityDim+1,'color','lightgrey');
      %if ~fc_tools.comp.isOctave(),alpha(0.1); end
      Gh.plotmesh(1,'color','lightgrey');
      Gh.plotmesh(entityDim,'interfaces',true);
    end
    if entityDim==Gh.d
      for i=1:n
        me=sElts.me(:,sElts.part_lab==tags(i));
        [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(Gh.q,me,Gh.toGlobal);
        fc_graphics4mesh.plotmesh(q,me,'color',Colors(i,:),'DisplayName',num2str(tags(i)),'inlegend',true)
      end
      plt=1;
    else
      plt=0;
      for i=1:n
        plt=plt+plot_extractPartition(Gh,entityDim,tags(i),Colors(i,:));
      end
      if plt==0,close(hh);end 
    end
    if plt>0
      title(sprintf('Partition elements, entityDim:%d',entityDim))
      legend(gca,'show')  % gca,'show' needs for older Matlab version, 2016b for ex.
      axis image
      axis off
      if Gh.dim==3,view(3);end
    end
  end
end  

function plt=plot_extractPartition(Gmsh,d,plabel,Color) % part of the mooGmsh extractPartition function
  inlegend=true;
  k=find(cell2mat({Gmsh.sElts(:).d})==d);
  if ~isempty(k)
    I=find(Gmsh.sElts(k).nb_parts>=1);
    if ~isempty(I)
      J=find(Gmsh.sElts(k).part_lab(I,1)==plabel);
      IJ=I(J);
      nme=length(IJ);
      if nme>0
        me=Gmsh.sElts(k).me(:,IJ);
        [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(Gmsh.q,me,Gmsh.toGlobal);
        fc_graphics4mesh.plotmesh(q,me,'color',Color,'DisplayName',num2str(plabel),'inlegend',inlegend)
        inlegend=false;
      else
        I=1:Gmsh.sElts(k).nme;
        for j=1:d+1
          [C,IA,IB]=intersect(Gmsh.sElts(k).me(j,:),Gmsh.toGlobal);
          I=intersect(I,IA);
        end
        I=ismember(Gmsh.sElts(k).me(1,:),Gmsh.toGlobal);
        for j=2:d+1
          I=I & ismember(Gmsh.sElts(k).me(j,:),Gmsh.toGlobal);
        end
        nme=length(I);
        if nme>0
          me=Gmsh.sElts(k).me(:,I);
          %mel=Gmsh.sElts(k).geo_lab(I);
          [q,me,toGlobal]=fc_oogmsh.msh.FromGlobalToLocal(Gmsh.q,me,Gmsh.toGlobal);
          fc_graphics4mesh.plotmesh(q,me,'color',Color,'DisplayName',num2str(plabel),'inlegend',inlegend)
          inlegend=false;
        end
      end
    end
  end
  plt=~inlegend;
end
