function Entities=read_Entities(fid,varargin)
  p = inputParser;
  p.addParameter('verbose',false,@islogical);
  p.parse(varargin{:});
  verbose=p.Results.verbose;
  flag=fc_oogmsh.msh.find_section(fid,'$Entities',varargin{:});
  if ~flag,error('Unable to find $Entities');end
  R=fscanf(fid,'%d',4);fgetl(fid);
  numPoints=R(1);numCurves=R(2);numSurfaces=R(3);numVolumes=R(4);
  
  % Read point entities
  pE=struct('tag',[],'min',[],'max',[],'numPhysicalTags',[],'PhysicalTags',[],'Point',[]);
  pE=repmat(pE,1,numPoints);
  for i=1:numPoints
    pE(i)=read_Entity(fid,pE(i));
    pE(i).Point=pE(i).max;
  end
  
  % read curve entities
  pC=struct('tag',[],'min',[],'max',[],'numPhysicalTags',[],'PhysicalTags',[],'numBoundingPoints',[],'PointTags',[]);
  pC=repmat(pC,1,numCurves);
  for i=1:numCurves
    pC(i)=read_Entity(fid,pC(i));
    pC(i).numBoundingPoints=fscanf(fid,'%d',1);
    if pC(i).numBoundingPoints>0,pC(i).PointTags=fscanf(fid,'%d',pC(i).numBoundingPoints)';end
  end
  % read surface entities
  pS=struct('tag',[],'min',[],'max',[],'numPhysicalTags',[],'PhysicalTags',[],'numBoundingCurves',[],'CurveTags',[]);
  pS=repmat(pS,1,numSurfaces);
  for i=1:numSurfaces
    pS(i)=read_Entity(fid,pS(i));
    pS(i).numBoundingCurves=fscanf(fid,'%d',1);
    if pS(i).numBoundingCurves>0,pS(i).CurveTags=fscanf(fid,'%d',pS(i).numBoundingCurves)';end
  end
  % read volume entities
  pV=struct('tag',[],'min',[],'max',[],'numPhysicalTags',[],'PhysicalTags',[],'numBoundingSurfaces',[],'SurfaceTags',[]);
  pV=repmat(pV,1,numVolumes);
  for i=1:numVolumes
    pV(i)=read_Entity(fid,pV(i));
    pV(i).numBoundingSurfaces=fscanf(fid,'%d',1)';
    if pV(i).numBoundingSurfaces>0,pV(i).SurfaceTags=fscanf(fid,'%d',pV(i).numBoundingSurfaces)';end
  end
  text = fscanf(fid,'%s',1);
  if ( text == -1 ) | ~strcmp(strtok(text),'$EndEntities') 
    error('Trouble when reading file in section $Entities:\n  ->  %s',text)
  else
    if verbose,fprintf('Reading $Entities section seem OK\n');end
  end
  Entities=struct('numPoints',numPoints, 'Points',pE, ...
                  'numCurves',numCurves, 'Curves',pC, ...
                  'numSurfaces',numSurfaces, 'Surfaces',pS, ...
                  'numVolumes',numVolumes, 'Volumes',pV);
end

function pEntity=read_Entity(fid,pEntity)
  R=fscanf(fid,'%f',8);
  pEntity.tag=R(1);
  pEntity.min=R(2:4);
  pEntity.max=R(5:7);
  pEntity.numPhysicalTags=R(8);
  if R(8)>0,pEntity.PhysicalTags=fscanf(fid,'%d',R(8))';end
end
