function Th=olympic_rings(N,varargin)
  ip = inputParser();
  %ip.KeepUnmatched=true;
  ip.PartialMatching=false;
  ip.addParameter('verbose',0,@isnumeric); % fine mesh refinement number
  ip.addParameter('delete',true,@islogical); % delete msh file
  ip.addParameter('h',0.5,@isnumeric); % see geofile
  ip.addParameter('R',3,@isnumeric); % see geofile
  ip.addParameter('renumbering',true,@islogical); %
  ip.addParameter('tol',1e-12,@isnumeric); %
  ip.parse(varargin{:});
  iR=ip.Results;
  %varargin=fc_tools.utils.deleteCellOptions(varargin,ip.Parameters);
  Loptions={sprintf('-setnumber h %g',iR.h),sprintf('-setnumber R %g',iR.R)};
  geofile=fc_oogmsh.get_geo(2,2,'olympic_ringsOC.geo');
  meshfile=fc_oogmsh.buildmesh2d(geofile,N,'Loptions',Loptions,'verbose',iR.verbose);
  Th=fc_simesh.siMesh(meshfile,'isPhysLabel',false);
  Th.other.parameters=struct('N',N,'h',iR.h,'R',iR.R);
  if iR.renumbering
    R=iR.R;h=iR.h;
    r=R-h/2;
    tol=iR.tol;
    % Disk(1) and Disk(11) (see geofile)
    cx=0;cy=0;
    fun=@(q) all( abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -R^2 ) < tol | abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -r^2 ) < tol);
    d=1; % d-simplex
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThlab(idx); newlabel=1;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: (%d-simplex) numbering geometric labels to physical label : %s   -> %d\n',mfilename(),d,num2str(oldlabel),newlabel);end

    % Disk(2) and Disk(12) (see geofile)
    cx=-2*R-h;cy=0;
    fun=@(q) all( abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -R^2 ) < tol | abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -r^2 ) < tol);
    d=1; % d-simplex
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThlab(idx); newlabel=2;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: (%d-simplex) numbering geometric labels to physical label : %s   -> %d\n',mfilename(),d,num2str(oldlabel),newlabel);end

    % Disk(3) and Disk(13) (see geofile)
    cx=-R-h/2;cy=-R;
    fun=@(q) all( abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -R^2 ) < tol | abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -r^2 ) < tol);
    d=1; % d-simplex
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThlab(idx); newlabel=3;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: (%d-simplex) numbering geometric labels to physical label : %s   -> %d\n',mfilename(),d,num2str(oldlabel),newlabel);end

    % Disk(4) and Disk(14) (see geofile)
    cx=R+h/2;cy=-R;
    fun=@(q) all( abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -R^2 ) < tol | abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -r^2 ) < tol);
    d=1; % d-simplex
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThlab(idx); newlabel=4;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: (%d-simplex) numbering geometric labels to physical label : %s   -> %d\n',mfilename(),d,num2str(oldlabel),newlabel);end

    % Disk(5) and Disk(15) (see geofile)
    cx=2*R+h;cy=0;
    fun=@(q) all( abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -R^2 ) < tol | abs( (q(1,:)-cx).^2+(q(2,:)-cy).^2 -r^2 ) < tol);
    d=1; % d-simplex
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThlab(idx); newlabel=5;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: (%d-simplex) numbering geometric labels to physical label : %s   -> %d\n',mfilename(),d,num2str(oldlabel),newlabel);end

    Th.set_pcolors(); % need to set new physical color to each sTh elements
  end
  if iR.delete
    if iR.verbose,fprintf('%s: delete mesh file :\n   -> %s\n',mfilename(),meshfile);end
    delete(meshfile)
  end
end
