function Th=olympic_rings(N,varargin)
  ip = inputParser();
  %ip.KeepUnmatched=true;
  ip.PartialMatching=false;
  ip.addParameter('verbose',0,@isnumeric); % fine mesh refinement number
  ip.addParameter('delete',true,@islogical); % delete msh file
  ip.addParameter('h',0.5,@isnumeric); % see geofile
  ip.addParameter('R',3,@isnumeric); % see geofile
  ip.addParameter('tol',1e-12,@isnumeric); % see geofile
  ip.addParameter('renumbering',true,@islogical); %
  ip.parse(varargin{:});
  iR=ip.Results;
  %varargin=fc_tools.utils.deleteCellOptions(varargin,ip.Parameters);
  Loptions={sprintf('-setnumber h %g',iR.h),sprintf('-setnumber R %g',iR.R)};
  geofile=fc_oogmsh.get_geo(2,2,'olympic_ringsv2OC.geo');
  meshfile=fc_oogmsh.buildmesh2d(geofile,N,'Loptions',Loptions,'verbose',iR.verbose);
  Th=fc_simesh.siMesh(meshfile,'isPhysLabel',false);
  Th.other.parameters=struct('N',N,'h',iR.h,'R',iR.R);
  if iR.renumbering
    R=iR.R;h=iR.h;tol=iR.tol;
    r=R-h/2;
    C=[0,0; -2*R-h,0; -R-h/2,-R; R+h/2,-R; 2*R+h,0]; %C(i,:) center of i-th ring

    for i=1:5 % ring i : Disk(i) and Disk(10+i) (see geofile)
      inring{i}=@(q) ( (q(1,:)-C(i,1)).^2+(q(2,:)-C(i,2)).^2 <= R^2+tol ) & ( (q(1,:)-C(i,1)).^2+(q(2,:)-C(i,2)).^2 >= r^2-tol );
      fun=@(q) all( inring{i}(q) );
      for d=1:2 % d-simplex
        idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
        oldlabel=Th.sThphyslab(idx); newlabel=i;
        Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
        if iR.verbose,fprintf('%s: (%d-simplex) renumbering physical labels : %s   -> %d\n',mfilename(),d,num2str(oldlabel),newlabel);end
      end
    end

    fun=@(q) all( inring{1}(q) & inring{3}(q) ) | all( inring{2}(q) & inring{3}(q) | ...
             all( inring{1}(q) & inring{4}(q) ) | all( inring{4}(q) & inring{5}(q) )   ) ;
    d=1;
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThphyslab(idx); newlabel=0;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: renumbering physical labels : %s   -> %d\n',mfilename(),num2str(oldlabel),newlabel);end

    d=2;
    idx=fc_simesh.methods_siMesh.find_elements_in(Th,d,fun);
    oldlabel=Th.sThphyslab(idx); newlabel=10;
    Th=fc_simesh.methods_siMesh.set_physical_label(Th,d,idx,newlabel,'set_pcolors',false);
    if iR.verbose,fprintf('%s: renumbering physical labels : %s   -> %d\n',mfilename(),num2str(oldlabel),newlabel);end

    Th.set_pcolors(); % need to set new physical color to each sTh elements
  end
  if iR.delete
    if iR.verbose,fprintf('%s: delete mesh file :\n   -> %s\n',mfilename(),meshfile);end
    delete(meshfile)
  end
end
