classdef siMesh < handle 
  
  properties %(SetAccess = protected)
    d =0;
    dim=0;
    sTh={};
    nsTh=0;
    toGlobal=[];
    toParent=[];
%    nqGlobal=0;;
    sThsimp=[];
    sThlab=[];
    sThcolors=[];
    bbox=[]; % Bounding box
    sThgeolab=[];
    sThphyslab=[];
    sThpartlabs={};
    sThboundlabs={}; % 
    sThPhysicalTags={};
    %PartitionedEntity={}; % used to "debug"
    %sElts=[];
    % Ms=[];
    nq=0;
    nqParents = []; 
    toParents ={};
    other=[];
  end
  
  methods
  
    function self = siMesh(cFileName,varargin)
      if nargin==0, return;end
      if nargin>0
        assert(ischar(cFileName),'First parameter must be file name (char)')
        p = inputParser;
        p.addParamValue('dim', 2, @isfloat );
        p.addParamValue('d', [] );
        %p.addParamValue('perm',true); % permutation of local points : (false : as in (medit format))
        p.addParamValue('color',true);
        p.addParamValue('format','gmsh',@(x) ismember(x,{'medit','gmsh','freefem','triangle'}));
        p.addParamValue('isPhysLabel',true,@islogical);
        p.addParamValue('normals',true,@islogical);
        p.addParamValue('trans',[]);
        p.parse(varargin{:});
        R=p.Results;
        if isempty(R.d),  R.d=[R.dim:-1:(R.dim-1)];end
        switch R.format
          case('medit')
                  self=fc_simesh.read.readMedit(cFileName,R.dim,R.d);
          case('gmsh')
                  self=fc_simesh.read.readgmsh(cFileName,R.dim,R.d,R.isPhysLabel);
          case('freefem')
                  self=fc_simesh.read.readfreefem(cFileName,R.dim,R.d);
          case('triangle')
                  self=fc_simesh.read.readtriangle(cFileName,R.dim);
        end
        if R.color, self.set_colors();self.set_pcolors(); end
        self.nsTh=length(self.sTh);
        self.setTags();
        self.set_bbox();
        self.other.meshfile=cFileName;
        self.other.format=R.format;
        if ~isempty(R.trans)
          self.transform(R.trans);
        end
        self.other.trans=R.trans;
        if R.normals
          fc_simesh.methods_siMesh.BoundaryNormals(self);
        end
      end
    end 
    
%     function transform(self,trans)
%       for k=1:self.np
%         self.Th{k}.transform(trans);
%       end
%       self.set_bbox();
%     end

    function self=set_bbox(self)
      self.bbox=self.sTh{1}.bbox;
      for k=2:self.nsTh
        bb=self.sTh{k}.bbox;
        for i=1:self.dim
          self.bbox(2*i-1)=min(self.bbox(2*i-1),bb(2*i-1));
          self.bbox(2*i)=max(self.bbox(2*i),bb(2*i));
        end
      end
    end
    
    function self=set_colors(self,varargin)
      self.sThcolors=zeros(self.nsTh,3);
      for d=0:self.d
        Ilab=self.find(d);
        if ~isempty(Ilab)
          self.sThcolors(Ilab,:)=fc_tools.graphics.selectColors(length(Ilab),varargin{:});
        end
        for jlab=Ilab
          self.sTh{jlab}.color=self.sThcolors(jlab,:);
        end
      end
    end

    function self=set_pcolors(self,varargin) % set physical colors
      for d=0:self.d
        [idx,pLab]=self.find_physicals(d);
        plab=unique(pLab);
        colors=fc_tools.graphics.selectColors(length(plab),varargin{:});
        k=1;
        for pl=plab
          ii=find(self.sThphyslab(idx)==pl);
          for i=idx(ii)
            self.sTh{i}.pcolor=colors(k,:);
          end
          k=k+1;
        end
      end
    end
    
    function labs=get_labels(self,d)
       labs=self.sThlab(self.find(d));
    end
    
    function self=setTags(self)
      for i=1:self.nsTh
        Tag=sprintf('d:%d, label : %d\nnq=%d, nme=%d',self.sTh{i}.d,self.sTh{i}.label,self.sTh{i}.nq,self.sTh{i}.nme);
        self.sTh{i}.Tag=Tag;
      end
    end
    
    
    function self=set_d(self)
      self.d=max(self.sThsimp);
    end
    
    function self=set_toGlobal(self)
      R=cellfun(@(x) x.toGlobal,self.sTh,'UniformOutput',false);
      self.toGlobal=unique([R{:}]);
      self.nq=length(self.toGlobal);
    end
    
    function self=set_Parents(self)
      for i=1:self.nsTh
        [C,ia,ib]=intersect(self.sTh{i}.toGlobal,self.toGlobal);
        self.sTh{i}.toParent=ib'; % to delete in future version
        self.sTh{i}.toParents{1}=ib';
        self.sTh{i}.nqParent=self.nq; % to delete in future version
        self.sTh{i}.nqParents(1)=self.nq;
        self.sTh{i}.nqParents(2)=self.sTh{i}.nqGlobal;
        self.sTh{i}.toParents{2}=self.toGlobal;
      end
    end
    
    function toG=get_toGlobal(self,d,labels)
      idx=self.find(d,labels);
      C=cellfun(@(x) x.toGlobal,{self.sTh{idx}},'UniformOutput',false);
      toG=unique([C{:}]);
    end
    
    function self=AddCopysiMeshElt(self,p1Elmt)
      I=intersect(p1Elmt.toGlobal,self.toGlobal); % On verifie que le p1Elmt appartient au P1Mesh
      if length(I)==length(p1Elmt.toGlobal)
        
        N=self.nsTh+1;
        self.nsTh=N;
        self.sTh{N}=p1Elmt.copy();
        
        self.sThsimp(N)=p1Elmt.d;
        self.sThlab(N)=p1Elmt.label;
      end
    end    
    
    function Th=copy(self)
      Th=fc_simesh.siMesh();
      Th.d=self.d;
      Th.dim=self.dim;
      Th.nsTh=self.nsTh;
      Th.sTh=cell(1,Th.nsTh);
      for i=1:Th.nsTh, Th.sTh{i}=self.sTh{i}.copy();end
      Th.toGlobal=self.toGlobal;
      Th.toParent=self.toParent;
      Th.sThsimp=self.sThsimp;
      Th.sThlab=self.sThlab;
      Th.sThcolors=self.sThcolors;
      Th.bbox=self.bbox;
      Th.sThgeolab=self.sThgeolab;
      Th.sThphyslab=self.sThphyslab;
      Th.sThboundlabs=self.sThboundlabs;
      Th.nq=self.nq;
      Th.nqParents=self.nqParents;
      Th.toParents=self.toParents;
      Th.other=self.other;
    end
    
    function self=AddsiMeshElt(self,ooElmt)
      if ~isempty(self.sTh)
        I=intersect(ooElmt.toGlobal,self.toGlobal); % On verifie que le ooElmt appartient au ooMesh
        assert( length(I)==length(ooElmt.toGlobal) );
        N=self.nsTh+1;
      else
        N=1;
        self.toGlobal=ooElmt.toGlobal;
        self.nq=ooElmt.nq;
      end
        self.nsTh=N;
        self.sTh{N}=ooElmt;
        self.sThsimp(N)=ooElmt.d;
        self.sThlab(N)=ooElmt.label;
    end
    
    function self=AddsiMeshElts(self,d,q,nme,me,mel,varargin)
      p = inputParser;
      p.addParamValue('geolab', [] );
      p.addParamValue('partlab', {} );
      p.parse(varargin{:});
      R=p.Results;
      labels=unique(mel);
      k=self.nsTh;
      nq=size(q,2);
      partlab={};
      geolab=[];
      if d==self.d
        %self.toGlobal=unique([self.toGlobal,unique(me)']);
        self.toParent=unique([self.toParent,unique(me)']);
        self.toGlobal=self.toParent;
        self.toParents{1}=self.toParent;
        self.nqParents(1)=size(q,2);
      end
      for i=1:length(labels)
        I=find(mel==labels(i));
        ME=me(:,I);
        if ~isempty(R.geolab), geolab=R.geolab(I);end
        if ~isempty(R.partlab), partlab=R.partlab(I);end
        indQ=unique(ME);
        Q=q(:,indQ);
        lQ=1:length(indQ);
        J=zeros(nq,1);
        J(indQ)=lQ;
        if(d==0) %correctif a faire si d=0 (programmation a l'arrache! To improve)
          MELoc=J(ME)';
        else
          MELoc=J(ME);
          end
        self.sTh{k+i}=fc_simesh.siMeshElt(self.dim,d,labels(i),Q,MELoc,indQ',nq,geolab,partlab);
        self.sThsimp(k+i)=d;
        self.sThlab(k+i)=labels(i);
      end
      self.nsTh=self.nsTh+length(labels);
    end    
 
    function Ba=barycenters(self,varargin)
      % Barycenters of selected elements
      p = inputParser; 
      p.addParamValue('d',self.d,@isscalar);
      p.addParamValue('labels',[]); % select labels to draw
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      if isempty(idxlab), return;end
      j=1;
      Ba=zeros(self.dim,length(idxlab));
      for ilab=[idxlab]
        Ba(:,j)=mean(self.sTh{ilab}.barycenters(),2);
        j=j+1;
      end
    end
 
    function obj=DeletesiMeshElt(self,idx)
      % idx : partition label
      obj=siMesh();
      obj.d=self.d;
      obj.dim=self.dim;
      %obj.nsTh=self.nsTh-1;
      obj.toGlobal=self.toGlobal;
      I=setdiff(1:self.nsTh,idx);
      for i=[I]
        obj=obj.AddCopysiMeshElt(self.sTh{i});
      end
      obj.set_colors();
      obj.setTags();
      obj.set_bbox();
      
      I=[];for i=1:obj.nsTh,I=[i,obj.sTh{i}.toGlobal];end
      [toGlobal,ia,ic]=unique(I);
      obj.nq=length(toGlobal);
      obj.toGlobal=1:obj.nq;
      J=1:obj.nq; 
      B=sparse(toGlobal,ones(1,obj.nq),J);
      for i=1:obj.nsTh
        obj.sTh{i}.toGlobal=full(B(obj.sTh{i}.toGlobal,1))';
        obj.sTh{i}.nqGlobal=obj.nq;
        obj.sTh{i}.toParent=obj.sTh{i}.toGlobal;
        obj.sTh{i}.nqParent=obj.nq;
      end
    end 
    

    function disp(self)
    % FUNCTION disp, method function of the siMesh class.
    %   Displays informations on the siMesh object.
    %
    % Copyright (C) 2017-2023 F. Cuvelier
      fc_tools.utils.disp_object(self);
    end    
    
    function u=eval(self,fun,varargin)
    % Eval fun on siMesh object and return an array with shape (self.nq,1)
    %        corresponding on the evaluate of fun on each mesh vertices (if fun is not a 
    %        list and not None)
    %        
    %        fun could be:
    %          - a scalar
    %          - a function handle
    %          - an array 
    %          - a cell array where each element could be on of the previous type
      if isnumeric(fun)
        if size(fun,1)==self.nq %&& size(fun,2)==1
          u=fun;
          return
        end
        if size(fun,2)==self.nq %&& size(fun,1)==1
          u=fun.';
          return
        end
      end

      p = inputParser; 
      p.addParamValue('level',0);
      p.addParamValue('d',self.d,@isscalar);
      p.addParamValue('labels',[]);
      p.addParamValue('cell',false,@islogical);
      p.parse(varargin{:});
      R=p.Results;
      
      if iscell(fun)
        assert(size(fun,1)==1,'fun must be an 1-by-m cell array, given %d-by-%d ',size(fun))
        m=length(fun);
        if R.cell
          u=cell(m,1);
          for i=1:m
            u{i}=self.eval(fun{i},varargin{:});
          end
        else
          u=zeros(self.nq,m);
          for i=1:m
            u(:,i)=self.eval(fun{i},varargin{:});
          end
        end
        return
      end
      
      assert(ismember(R.level,0:length(self.nqParents)))
      
      if isempty(R.labels), idxlab=self.find(self.d);else idxlab=self.find(R.d,R.labels);end
      
      if R.level==0
        u=zeros(self.nq,1);
      else
        u=zeros(self.nqParents(R.level),1);
      end
      for i=idxlab
        u(self.sTh{i}.toParents{R.level+1})=self.sTh{i}.eval(fun);
      end  
    end    
    
    function obj=extractPartition(self,plabel)
      % label : partition label
      i=self.find(self.d,plabel);
      if isempty(i), obj=[];return; end
      obj=siMesh();
      obj.d=self.d;
      obj.dim=self.dim;
      l=1; 
      obj.sTh{l}=self.sTh{i};
      obj.nsTh=1;
      obj.sThsimp(l)=obj.d;
      obj.sThlab(l)=plabel;
      q=obj.sTh{l}.q;
      nq=obj.sTh{l}.nq;
      obj.nq=obj.sTh{l}.nq;
      I=self.find(self.d-1);
      for k=1:length(I)
        i=I(k);
        if self.sTh{i}.label>0
          if sum(cellfun(@(x) size(x,1),self.sTh{i}.partlab)) == length(self.sTh{i}.partlab)
            partlab=cellfun(@(x) x(1),self.sTh{i}.partlab);
            P=find(partlab==label);
            if ~isempty(P)
              me=self.sTh{i}.me(:,P);
              mel=self.sTh{i}.label*int32(ones(1,length(P)));
              obj.AddElements(self.d-1,self.sTh{i}.q,length(mel),me,mel,self.sTh{i}.indve);
            end
          end
        else % fictif (ghost)
          Ql=self.sTh{i}.q(:,self.sTh{i}.me(:,1));
          % Check if all Ql points are in q? Un peu pourri comme methode! => a ameliorer
          isIn=true;r=1;nQl=size(Ql,2);
          while ((isIn)&&(r<=nQl))
            S=sqrt(sum((q-Ql(:,r)*ones(1,nq)).^2,1));
            A=find(S<1e-6);
            if isempty(A), isIn=false;end
            r=r+1;
          end
          if isIn
            obj.nsTh=obj.nsTh+1;
            obj.sTh{obj.nsTh}=self.sTh{i};
            obj.sThsimp(obj.nsTh)=self.d-1;
            obj.sThlab(obj.nsTh)=self.sTh{i}.label;
          end
        end
      end
    end    
 
    function obj=extractSubDomain(self,label)
      % label : partition label
      i=self.find(self.d,label);
      if isempty(i), obj=[];return; end
      obj=PkMesh();
      obj.d=self.d;
      obj.dim=self.dim;
      obj.order=self.order;
      l=1; 
      obj.sTh{l}=self.sTh{i};
      obj.nsTh=1;
      obj.sThsimp(l)=obj.d;
      obj.sThlab(l)=label;
      q=obj.sTh{l}.q;
      nq=obj.sTh{l}.nq;
      obj.nq=obj.sTh{l}.nq;
      I=self.find(self.d-1);
      for k=1:length(I)
        i=I(k);
        if self.sTh{i}.label>0
          if sum(cellfun(@(x) size(x,1),self.sTh{i}.partlab)) == length(self.sTh{i}.partlab)
            partlab=cellfun(@(x) x(1),self.sTh{i}.partlab);
            P=find(partlab==label);
            if ~isempty(P)
              me=self.sTh{i}.me(:,P);
              mel=self.sTh{i}.label*int32(ones(1,length(P)));
              obj.AddElements(self.d-1,self.sTh{i}.q,length(mel),me,mel,self.sTh{i}.indve);
            end
          end
        else % fictif (ghost)
          Ql=self.sTh{i}.q(:,self.sTh{i}.me(:,1));
          % Check if all Ql points are in q? Un peu pourri comme methode! => a ameliorer
          isIn=true;r=1;nQl=size(Ql,2);
          while ((isIn)&&(r<=nQl))
            S=sqrt(sum((q-Ql(:,r)*ones(1,nq)).^2,1));
            A=find(S<1e-6);
            if isempty(A), isIn=false;end
            r=r+1;
          end
          if isIn
            obj.nsTh=obj.nsTh+1;
            obj.sTh{obj.nsTh}=self.sTh{i};
            obj.sThsimp(obj.nsTh)=self.d-1;
            obj.sThlab(obj.nsTh)=self.sTh{i}.label;
          end
        end
      end
    end
    
    function u=feval(self,fun,varargin)
    % Eval fun on siMesh object and return an array with shape (self.nq,1)
    %        corresponding on the evaluate of fun on each mesh vertices (if fun is not a 
    %        list and not None)
    %        
    %        fun could be:
    %          - a function handle
    %          - a cell array of function handles

      if iscell(fun)
        assert(size(fun,1)==1,'fun must be an 1-by-m cell array, given %d-by-%d ',size(fun))
        m=length(fun);
        u=zeros(self.nq,m);
        for i=1:m
          assert(fc_tools.utils.isfunhandle(fun{i}),'%d-th cell must be a function handle',i)
          u(:,i)=self.feval(fun{i},varargin{:});
        end
        return 
      end
      
      assert(fc_tools.utils.isfunhandle(fun))
      assert( ismember(nargin(fun),[1,self.dim]) ,'Number of function arguments must be 1 or %d',self.dim)

      p = inputParser; 
      p.addParamValue('d',self.d,@isscalar);
      p.addParamValue('level',0,@isscalar); % mainly used with partition meshes?
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      assert(ismember(R.level,0:length(self.nqParents)))
      
      if isempty(R.labels), idxlab=self.find(self.d);else idxlab=self.find(R.d,labels);end
      
      if R.level==0
        u=zeros(self.nq,1);
      else
        u=zeros(self.nqParents(R.level),1);
      end
      for i=idxlab
        u(self.sTh{i}.toParents{R.level+1})=self.sTh{i}.feval(fun);
      end
    end    
    
    function varargout=find_physicals(this,varargin)
    % usage:
    % idx=Th.find_physicals(d); or idx=find_physicals(Th,d);
    %    returns indices of d-simplex physical "elements" with physical labels given by Th.sThphyslab(idx)
    % [idx,plab]=Th.find_physicals(d);
    %    also returns physical labels
    % idx=Th.find_physicals(d, plabs);
    %    returns indices of d-simplex physical "elements" with physical labels in plabs
    % [idx,plab]=Th.find_physicals(d,plabs);
    %    also returns physical labels
    % examples:
    % [idx,pval]=Th.find_physicals(1);
    % idx=Th.find_physicals(1,[1,14]);
    % [idx,pval]=Th.find_physicals(1,[1,14]);
    %
    assert(ismember(nargin-1,[1,2]),'mismatch input arguments!')
    ind=find(this.sThsimp==varargin{1} & this.sThphyslab>0 );
    plab=this.sThphyslab(ind);
    varargout{1}=ind;
    if nargout==2,varargout{2}=plab;end
    switch nargin
        case 2
          return
        case 3
          labs=varargin{2};
          if isempty(labs),return;end
          labs=labs(ismember(labs,plab)); % uniquement les "vrais" labels physiques
          I=[];
          for i=1:length(labs)
            I=[I,ind(find(plab==labs(i)))];
          end
          varargout{1}=I;
          if nargout==2,varargout{2}=this.sThphyslab(I);end
        otherwise
          varargout{1}=[];
          warning('unknown number of arguments')
      end
    end
    
    function varargout=find(this,varargin)
      switch nargin
        case 2
          % d=varargin{1}
          ind=find(this.sThsimp==varargin{1});
          varargout{1}=ind;
          if nargout==2,varargout{2}=this.sThlab(ind);end
        case 3
          %ind=find((this.sThsimp==varargin{1}) & (this.sThlab==varargin{2}));
          I=find(this.sThsimp==varargin{1});
          % [J,c]=intersect(this.sThlab(I),varargin{2}); % Add test if labels are OK
          % intersect change order of elements
          [J,c]=ismember(varargin{2},this.sThlab(I));
          %keyboard
          if ~isempty(c)
           % keyboard
           if sum(c)==0
             ind=[];
            else
              ind=I(c(c~=0));
            end
          else
            ind=[];
          end
          varargout{1}=ind;
          if nargout==2,varargout{2}=this.sThlab(ind);end
    %        if length(ind)<length(varargin{2})
    %          str=sprintf(' %d,',setdiff(varargin{2},this.sThlab(I)));str(end)='.';
    %          warning('labels not found for %d-simplex sub-meshes : %s',varargin{1},str);
    %        end
        otherwise
          varargout{1}=[];
          warning('unknown number of arguments')
      end 
      %if isempty(ind), warning('Sub mesh not found ');end
    end    
    
    function ind=findphyslab(this,d,lab) 
    % return indices of <d>-simplicial elementary meshes with specify physical label <lab>
      I=find(this.sThsimp==d);
      J=find(this.sThphyslab(I)==lab);
      ind=I(J);
    end
    
    function labName=funLabName(self,d)
      if self.d==d
        if fc_tools.comp.isOctave()
          labName=@(i) ['\Omega_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\Omega_{',num2str(i),'}$'];
          labName=@(i) ['\Omega_{',num2str(i),'}'];
        end
        return;
      elseif self.d==(d+1)
      if fc_tools.comp.isOctave()
          labName=@(i) ['\Gamma_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\Gamma_{',num2str(i),'}$'];
          labName=@(i) ['\Gamma_{',num2str(i),'}'];
        end
        return;
      elseif self.d==(d+2)
        if fc_tools.comp.isOctave()
          labName=@(i) ['\partial\Gamma_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\partial\Gamma_{',num2str(i),'}$'];
          labName=@(i) ['\partial\Gamma_{',num2str(i),'}'];
        end
        return
      elseif self.d==(d+3)
        if fc_tools.comp.isOctave()
          labName=@(i) ['\partial^2\Gamma_{',num2str(i),'}'];
        else
          %labName=@(i) ['$\partial\Gamma_{',num2str(i),'}$'];
          labName=@(i) ['\partial^2\Gamma_{',num2str(i),'}'];
        end
        return
      end
      labName=[];
      return
    end    
    
    function cl=get_cl(self)
    % get characteristic length
      i=1:self.dim;
      cl=max(self.bbox(2*i)-self.bbox(2*i-1));
    end
    
    function h=get_h(self)
      h=0;
      for i=1:self.nsTh
        h=max(h,self.sTh{i}.h);
      end
    end    

    function [q,me,toglobal,varargout]=get_mesh(self,varargin)
      p = inputParser;
      p.addParamValue('d',self.d);
      p.addParamValue('labels',[]);
      p.addParamValue('level',1);
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      level=R.level;
      me=[];q=[];toglobal=[];
      if isempty(idxlab)
        if ~isempty(R.labels)
          warning('No labels index found with d=%d and labels=%s',R.d,num2str(R.labels))
        else
          warning('No labels index found with d=%d',R.d)
        end
        return
      end
      %isGradBaCo=(nargout==4) && (R.d==self.d);
      %if isGradBaCo,GradBaCo=[];end
      
      toglobal_level=[];toglobal_loc=[];GradBaCo=[];normals=[];
      me=[];
      for ilab=[idxlab]
        sTh=self.sTh{ilab};
        if size(sTh.toParents{level},2)==1 % rustine pour Octave
          toglobal_level=[toglobal_level,sTh.toParents{level}'];
          toglobal_loc=[toglobal_loc,sTh.toParents{1}'];
        else
          toglobal_level=[toglobal_level,sTh.toParents{level}];
          toglobal_loc=[toglobal_loc,sTh.toParents{1}];
        end
        if isempty(me)
          me=reshape(sTh.toParents{1}(sTh.me),size(sTh.me)); % reshape needed when only one element
        else
          me=[me,reshape(sTh.toParents{1}(sTh.me),size(sTh.me))]; % reshape needed when only one element
        end
        %if isGradBaCo,GradBaCo=cat(1,GradBaCo,sTh.gradBaCo);end
        if nargout>=4,GradBaCo=cat(1,GradBaCo,sTh.gradBaCo);end
        if nargout==5,normals=cat(2,normals,sTh.normals);end
        %else
        %  me=[me,sTh.toParents{1}(sTh.me)];
        %end
      end
      [toglobal_level,IR,IL] = unique(toglobal_level); % toglobal(IL)==toglobal_loc and toglobal==toglobal_loc(IR)
      [toglobal,IR,IL] = unique(toglobal_loc); % toglobal(IL)==toglobal_loc and toglobal==toglobal_loc(IR)
      nq=length(toglobal);
      idx=zeros(1,self.nq);
      idx(toglobal)=1:nq;
      q=zeros(self.dim,nq);
      for ilab=[idxlab]
        sTh=self.sTh{ilab};
        q(:,idx(sTh.toParents{1}))=sTh.q;
      end
      me=idx(me);
      toglobal=toglobal_level;
      if size(me,1)~=R.d+1, me=me';end
      %if isGradBaCo, varargout{1}=GradBaCo;end
      if nargout>=4, varargout{1}=GradBaCo;end
      if nargout==5, varargout{2}=normals;end
    end    
    
    function nme=get_nme(self,varargin)
      p = inputParser;
      p.addParamValue('d',self.d);
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      nme=0;
      for ilab=[idxlab]
        nme=nme+self.sTh{ilab}.nme;
      end
    end 
    
    function nq=get_nq(self,varargin)
      p = inputParser;
      p.addParamValue('d',self.d);
      p.addParamValue('labels',[]);
      p.parse(varargin{:});
      R=p.Results;
      if isempty(R.labels) && R.d==self.d, nq=self.nq; return;end
      if isempty(R.labels), idxlab=self.find(R.d);else idxlab=self.find(R.d,R.labels);end
      if length(idxlab)==0, nq=0;return;end
      if length(idxlab)==1, nq=self.sTh{ilab}.nq;return;end
      
      me=[];q=[];toglobal=[]; 
      toglobal_loc=[];
      me=[];
      for ilab=[idxlab]
        sTh=self.sTh{ilab};
        toglobal_loc=[toglobal_loc,sTh.toParents{1}];
      end
      toglobal= unique(toglobal_loc); % toglobal(IL)==toglobal_loc and toglobal==toglobal_loc(IR)
      nq=length(toglobal);
    end    
    
    function info(self,varargin)
      % version simplifiée pour le moment
      p = inputParser; 
      p.addParamValue('variable',true,@islogical);
      p.addParamValue('verbose',0,@(x) ismember(x,0:2));
      p.parse(varargin{:});
      R=p.Results;
      if R.variable
        fprintf('Variable %s [%s object] :\n',inputname(1),class(self));
      end
      if R.verbose>0, fprintf('  dim=%d, d=%d\n',self.dim,self.d);end
      fprintf('  nq=%d, nme=%d\n',self.nq,self.get_nme());
      if R.verbose>0
        for d=self.d:-1:0
          I=self.find(d);
          if  isempty(I)
            fprintf('  %d-simplices : none\n',d);
          else
            fprintf('  %d-simplices : number %d, labels : ',d,length(I));
            fprintf('%d ',self.sThlab(I));fprintf('\n')
            if R.verbose>=2
              for k=I
                fprintf('%7d: nq=%d, nme=%d\n',self.sThlab(k),self.sTh{k}.nq,self.sTh{k}.nme);
              end
            end
          end
        end
      end
    end    
    
    function move(Th,u,dims)
      if nargin==2, dims=1:Th.dim;end
      % Move mesh
      dims=unique(dims);
      ldims=length(dims);
      dimmax=max([dims,Th.dim]);
      U=zeros(dimmax,Th.nq);
      if iscell(u)
        dimu=length(u);
        assert(ldims==dimu)
        for i=1:dimu
          if all(size(u{i})==[1,Th.nq])
            U(dims(i),:)=u{i};
          elseif all(size(u{i})==[Th.nq,1])
            U(dims(i),:)=u{i}.';
          else
          keyboard
            error('Incompatible dimension of the %d cell: %d-by-%d given, expected 1-by-%d or %d-by-1\n',i,size(u{i},1),size(u{i},2),Th.nq,Th.nq)
          end
        end
      else
        if all(size(u)==[ldims,Th.nq]) 
          U(dims,:)=u;
        elseif all(size(u)==[Th.nq,ldims])
          U(dims,:)=u.';
        else
          error('Incompatible dimension of the array: %d-by-%d given, expected %d-by-%d or %d-by-%d\n',size(u,1),size(u,2),ldims,Th.nq,Th.nq,ldims)
        end
      end
      for i=1:Th.nsTh
        Th.sTh{i}.move(U(:,Th.sTh{i}.toGlobal));
      end
      Th.dim=dimmax;
    end
    
    function remove(Th,d,labels)
      idx=Th.find(d,labels);
      I=setdiff(1:Th.nsTh,idx);
      if ~isempty(idx)
        Th.sTh={Th.sTh{I}};
        Th.nsTh=length(Th.sTh);
        Th.sThsimp=Th.sThsimp(I);
        Th.sThlab=Th.sThlab(I);
        Th.sThcolors=Th.sThcolors(I,:);
      end
    end
    
    function transform(Th,fun) % U dim-by-nq array
      for i=1:Th.nsTh
        Th.sTh{i}.transform(fun);
      end
      Th.set_bbox();
    end
    
    function obj=set_nq_old(obj)
      idxdom=obj.find(obj.d);
      if length(idxdom)==1
        obj.nq=obj.sTh{idxdom}.nq;
      elseif length(idxdom)>1
        toG=[];
        for idx=[idxdom]
          toG=[toG,obj.sTh{idx}.toGlobal];
        end
        obj.nq=length(unique(toG));
      else
        error('Unable to set nq')
      end
    end
    
    function obj=set_nq(obj)
      if isempty(self.toGlobal)
        self.set_toGlobal();
      end
      self.nq=length(self.toGlobal);
    end    
    
    function set_normals(Th) % Using GradBaCo but may be unefficient
      [~,me,toG,GradBaCo]=Th.get_mesh();
      me=toG(me);
      [Fh,beIn]=fc_meshtools.simplicial.getFaces(me);
      d=Th.d;
      P=nchoosek(1:(d+1),d);
      for i=1:size(P,1), Q(i,1)=setdiff(1:d+1,P(i,:)); end
      idx=Th.find(Th.d-1);
      for k=idx
        meloc=sort(Th.sTh{k}.toGlobal(Th.sTh{k}.me),1);
        [C,IA,IB]=intersect(meloc',Fh','rows');
        assert( size(meloc,2) == size(C,1)) 
        % meloc(:,IA) == Fh(:,IB)
        beInloc=beIn(IB,:);
        Fhloc=Fh(:,IB);
        [C,IA,IB]=intersect(meloc',Fhloc','rows');
        [I,J,K]=find(beInloc);
        [I,IJ]=unique(I);J=J(IJ);K=K(IJ); % when boundary is not a physcical boundary
        II=Q(K);
        idx=sub2ind(size(me),II,J);
        %
        %  me(il,i)= opposite index of the j-th edge in q array: il=II(l), i=I(l)  j=J(l) forall l
        %[me(:,J');Fhloc(:,I');me(idx)'] % normalement sur chaque colonne les 3 dernieres valeurs sont dans les 3 premieres
        G=GradBaCo(J,:,:);
        GG=zeros(Th.dim,length(J));
        for i=1:Th.d+1
          idx2=find(II==i);
          GG(:,IA(I(idx2)))=squeeze(G(idx2,i,:))';
        end
        Th.sTh{k}.normals=-GG./sqrt(sum(GG.^2,1));
      end
    
    end

    function varargout=vtkWrite(Th,FileName,varargin)
      p = inputParser;
      p.addParamValue('Values',{},@(x) iscell(x) || isnumeric(x) );
      p.addParamValue('Names',{},@(x) (iscell(x) || ischar(x)));
      p.addParamValue('idxsTh',[],@isnumeric);
      p.parse(varargin{:});
      U=p.Results.Values;
      names=p.Results.Names;
      idxsTh=p.Results.idxsTh;
      if isempty(idxsTh)
        idxsTh=Th.find(Th.d);
      else
        assert(isempty(setdiff(idxsTh,1:Th.nsTh)))
      end
      
      [pathstr,fname,fext] = fileparts(FileName);
      if exist([pathstr,filesep,fname])~=7,  mkdir([pathstr,filesep,fname]);end
      pvdfile=[pathstr,filesep,fname,'.pvd'];
      fid=fopen(pvdfile,'w');
      fprintf(fid,'<?xml version="1.0"?>\n');
      fprintf(fid,'<VTKFile type="Collection" version="1.0" byte_order="LittleEndian" header_type="UInt64">\n');
      fprintf(fid,'\t<Collection>\n');
      vtufile=[pathstr,filesep,fname,filesep,fname];
      svtufile=[fname,filesep,fname];
      idxfile=0;
      for i=[idxsTh]
        fullvtufile=sprintf('%s_%d.vtu',vtufile,idxfile);
        smallvtufile=sprintf('%s_%d.vtu',svtufile,idxfile);
        Th.sTh{i}.vtuWrite(fullvtufile,varargin{:});
        fprintf(fid,'\t\t<DataSet part="0"  file="%s"/>\n',smallvtufile);
        idxfile=idxfile+1;
      end
      
      fprintf(fid,'\t</Collection>\n');
      fprintf(fid,'</VTKFile>');
      fclose(fid);
      fprintf(' Paraview file %s created\n',pvdfile);
      if nargout==1, varargout{1}=pvdfile;end
    end    
    
    function [AdjElements,varargout]=AdjacentElements(Th,lab)
      % Returns global indices of adjacent elements to the (Th.d-1)-simplicial
      % submesh with label lab.
      %    AdjElements(k,1) is the global index of the adjacent element and 
      %    AdjElements(k,2) is the corresponding face index in the adjacent
      %    element
      idx=Th.find(Th.d-1,lab);
      assert(length(idx)==1)
      [q,me,toG,GradBaCo]=Th.get_mesh();

      [Fh,beIn]=fc_meshtools.simplicial.getFaces(me);
      Fh=sort(Fh',2);

      sTh=Th.sTh{idx};
      meb=sort(sTh.toGlobal(sTh.me),1); % boundary connectivity array
      [C,ia,ib]=intersect(meb',Fh,'rows');% So C = meb(ia,:) and C = Fh(ib,:).
      [I,J,K]=find(beIn(ib,:));
      
      A=zeros(size(I,1),1);
      A(I)=1:size(I,1);
      AdjElements=zeros(size(I,1),2);
      AdjElements(ia,:)=[J(A),K(A)];  % AdjElements(k,1) numero de l'élément adjacent, AdjElements(k,2) numero de l'arete dans l'élément adjacent
      if nargout==1,return;end
      
      [IndLocFaces,IndOpositePt]=fc_meshtools.simplicial.getIndLocFaces(Th.d);
      % meb(:,ia(I)) in me(:,J)
      meb=sTh.toGlobal(sTh.me);
      ME=me(:,J);
      GradBaCo=GradBaCo(J,:,:);
      nd=size(me,1)-1;np=length(J);
      MEF=zeros(nd,np);
      GradBaCoF=zeros(np,nd,Th.dim);
      IL=IndLocFaces(K,:)';
      for i=1:nd
        MEF(i,:)=ME(sub2ind([nd+1,np],IL(i,:),1:np));
          for j=1:Th.dim
            GradBaCoF(:,i,j)=GradBaCo(sub2ind(size(GradBaCo),1:np,IL(i,:),j*ones(1,np)));
          end
      end
      % MEF is meb(:,ia(I))
      % permutation des gradients si sommets sont permutés
      MEB=meb(:,ia(I));
      idx=find(sum(abs(MEB-MEF),1)~=0);
      P=perms(1:sTh.d+1);
      for i=1:size(P,1);
        idx=all(MEB==MEF(P(i,:),:),1);
        GradBaCoF(idx,:,:)=GradBaCoF(idx,P(i,:),:);
      end
      GradBaCoF(ia(I),:,:)=GradBaCoF;
      varargout{1}=GradBaCoF;
    end
      
    
    
%%% GRAPHICS    
    function varargout=plotmesh(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotmesh(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end

    function varargout=plotmesh_physicals(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotmesh_physicals(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function plotmeshinfo(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        fc_siplt.plotmeshinfo(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plot(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plot(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=scatter(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.scatter(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotiso(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotiso(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotquiver(self,V,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotquiver(self,V,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=slicemesh(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.slicemesh(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=slice(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.slice(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=sliceiso(self,u,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.sliceiso(self,u,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
%      function varargout=plotGradBaCo(self,varargin)
%        if fc_tools.utils.is_fcPackage('siplt')
%          varargout=cell(1,nargout);
%          [varargout{:}]=fc_siplt.plotGradBaCo(self,varargin{:});
%        else
%          if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
%          warning('Unable to load fc_siplt %s',name)
%        end
%      end
    
%      function varargout=plotElementsNumber(self,varargin)
%        if fc_tools.utils.is_fcPackage('siplt')
%          varargout=cell(1,nargout);
%          [varargout{:}]=fc_siplt.plotElementsNumber(self,varargin{:});
%        else
%          if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
%          warning('Unable to load fc_siplt %s',name)
%        end
%      end
    
    function varargout=plotnodes(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotnodes(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotnodesidx(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotnodesidx(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    function varargout=plotelementsidx(self,varargin)
      if fc_tools.utils.is_fcPackage('siplt')
        varargout=cell(1,nargout);
        [varargout{:}]=fc_siplt.plotelementsidx(self,varargin{:});
      else
        if fc_tools.comp.isOctave(), name='package';else, name='toolbox';end
        warning('Unable to load fc_siplt %s',name)
      end
    end
    
    
  end % methods 
end % classdef
